@@ -85,10 +85,6 @@ HTML_ACCESSIBLE_FACTORY (HTML_TYPE_BOX_TABLE_ACCESSIBLE, html_box_table_accessib
 HTML_ACCESSIBLE_FACTORY (HTML_TYPE_BOX_TEXT_ACCESSIBLE, html_box_text_accessible)
 #endif
 
-typedef enum {
-	HTML_VIEW_SCROLL_TO_TOP,
-	HTML_VIEW_SCROLL_TO_BOTTOM,
-} HtmlViewScrollToType;
 
 static gboolean
 set_adjustment_clamped (GtkAdjustment *adj, gdouble val)
@@ -111,7 +107,7 @@ set_adjustment_clamped (GtkAdjustment *adj, gdouble val)
     return FALSE;
 }
 
-static void
+void
 html_view_scroll_to_node (HtmlView *view, DomNode *node, HtmlViewScrollToType type)
 {
 	HtmlBox *box;
@@ -320,7 +316,10 @@ html_view_paint (HtmlView *view, GdkRectangle *area)
 {
 	/* Tell the root layout box to repaint itself */
 	if (view->painter && view->root) {
-		html_box_paint (view->root, view->painter, area, 0, 0);
+		/* Check that the document has not been deleted */
+		if (view->root->dom_node) {
+			html_box_paint (view->root, view->painter, area, 0, 0);
+		}
 	}
 }
 
@@ -361,9 +360,14 @@ html_view_insert_node (HtmlView *view, DomNode *node)
 	if (new_box) {
 
 		new_box->dom_node = node;
+		g_object_add_weak_pointer (G_OBJECT (node), (gpointer *) &(new_box->dom_node));
 
 		html_box_handle_html_properties (new_box, node->xmlnode);
 		
+		if (parent_box == NULL) {
+			if (!HTML_IS_BOX_ROOT (new_box))
+				parent_box = view->root;
+		}
 		if (parent_box == NULL) {
 			html_view_layout_tree_free (view, view->root);
 			if (view->document && view->document->focus_element)
@@ -652,10 +656,9 @@ html_view_realize (GtkWidget *widget)
 	 * GtkLayout uses the bg color for background but we want
 	 * to use base color.
 	 */	
- 	widget->style = gtk_style_copy (widget->style);
-	
- 	widget->style->bg[GTK_STATE_NORMAL] =
- 		widget->style->base[GTK_STATE_NORMAL];
+	widget->style = gtk_style_copy (widget->style);
+	widget->style->bg[GTK_STATE_NORMAL] = 
+		widget->style->base[GTK_STATE_NORMAL];
 	
 	if (GTK_WIDGET_CLASS (parent_class)->realize)
 		(* GTK_WIDGET_CLASS (parent_class)->realize) (widget);
@@ -1267,6 +1270,15 @@ html_view_get_type (void)
 	return type;
 }
 
+void
+html_view_style_set (GtkWidget *widget, GtkStyle *previous_style)
+{
+	if (previous_style) {
+ 		widget->style->bg[GTK_STATE_NORMAL] =
+ 			widget->style->base[GTK_STATE_NORMAL];
+	}
+}
+
 GtkWidget *
 html_view_new (void)
 {
@@ -1276,6 +1288,11 @@ html_view_new (void)
  	gtk_layout_set_hadjustment (GTK_LAYOUT (view), NULL);
  	gtk_layout_set_vadjustment (GTK_LAYOUT (view), NULL);
 
+	g_signal_connect (G_OBJECT (view),
+			  "style-set",
+			  G_CALLBACK (html_view_style_set),
+			  NULL);
+
 	return GTK_WIDGET (view);
 }
 
